@@ -46,11 +46,9 @@
 #include <nsPIDOMWindow.h>
 #include <nsIDOMWindow.h>
 #include <nsIDOMDocument.h>
-#if MOZILLA_BRANCH_MAJOR_VERSION < 16
-# include <nsIPrivateDOMEvent.h>
-#endif
+#include <nsIDOMElement.h>
+#include <nsIPrivateDOMEvent.h>
 #include <nsIDOMEventTarget.h>
-#include <mozilla/dom/Element.h>
 #include <nsIContent.h>
 
 #include <glib-object.h>
@@ -67,7 +65,6 @@
 #include "uWidgetAtoms.h"
 
 #include "uDebug.h"
-#include "compat.h"
 
 // XXX: Borrowed from content/xbl/src/nsXBLPrototypeHandler.cpp. This doesn't
 // seem to be publicly available, and we need a way to map key names
@@ -208,7 +205,7 @@ static struct keyCodeData gKeyCodes[] = {
 PRUint32
 uGlobalMenuItem::GetKeyCode(nsAString &aKeyName)
 {
-  nsCAutoString keyName;
+  nsAutoCString keyName;
   CopyUTF16toUTF8(aKeyName, keyName);
   ToUpperCase(keyName); // We want case-insensitive comparison with data
                         // stored as uppercase.
@@ -427,7 +424,7 @@ uGlobalMenuItem::SyncAccelFromContent()
   guint key = 0;
   mKeyContent->GetAttr(kNameSpaceID_None, uWidgetAtoms::key, keyStr);
 
-  nsCAutoString cKeyStr;
+  nsAutoCString cKeyStr;
   CopyUTF16toUTF8(keyStr, cKeyStr);
 
   if (!cKeyStr.IsEmpty()) {
@@ -462,16 +459,65 @@ uGlobalMenuItem::SyncStateFromCommand()
 {
   TRACETM()
 
+  nsAutoString checked;
+  if (mCommandContent && mCommandContent->GetAttr(kNameSpaceID_None,
+                                                  uWidgetAtoms::checked,
+                                                  checked)) {
+    LOGTM("Copying checked state from command node");
+    mContent->SetAttr(kNameSpaceID_None, uWidgetAtoms::checked, checked,
+                      true);
+  }
+}
+
+void
+uGlobalMenuItem::SyncLabelFromCommand()
+{
+  TRACETM()
+
+  nsAutoString label;
+  if (mCommandContent && mCommandContent->GetAttr(kNameSpaceID_None,
+                                                  uWidgetAtoms::label,
+                                                  label)) {
+    LOGTM("Copying label from command node");
+    mContent->SetAttr(kNameSpaceID_None, uWidgetAtoms::label, label,
+                      true);
+  }
+}
+
+void
+uGlobalMenuItem::SyncSensitivityFromCommand()
+{
+  TRACETM()
+
   if (mCommandContent) {
-    nsAutoString checked;
-    if (mCommandContent->GetAttr(kNameSpaceID_None, uWidgetAtoms::checked,
-                                 checked)) {
-      LOGTM("Copying checked state from command node");
-      mContent->SetAttr(kNameSpaceID_None, uWidgetAtoms::checked, checked, true);
+    if (mCommandContent->AttrValueIs(kNameSpaceID_None, uWidgetAtoms::disabled,
+                                     uWidgetAtoms::_true, eCaseMatters)) {
+      mContent->SetAttr(kNameSpaceID_None, uWidgetAtoms::disabled,
+                        NS_LITERAL_STRING("true"), true);
+    } else {
+      mContent->UnsetAttr(kNameSpaceID_None, uWidgetAtoms::disabled, true);
     }
   }
 }
 
+void
+uGlobalMenuItem::SyncStateFromContent()
+{
+  TRACETM()
+
+  if (!IsCheckboxOrRadioItem()) {
+    return;
+  }
+
+  SetCheckState(mContent->AttrValueIs(kNameSpaceID_None, uWidgetAtoms::checked,
+                                      uWidgetAtoms::_true, eCaseMatters));
+  dbusmenu_menuitem_property_set_int(mDbusMenuItem,
+                                     DBUSMENU_MENUITEM_PROP_TOGGLE_STATE,
+                                     IsChecked() ?
+                                     DBUSMENU_MENUITEM_TOGGLE_STATE_CHECKED : 
+                                      DBUSMENU_MENUITEM_TOGGLE_STATE_UNCHECKED);
+}
+
 void
 uGlobalMenuItem::SyncTypeAndStateFromContent()
 {
@@ -496,13 +542,7 @@ uGlobalMenuItem::SyncTypeAndStateFromContent()
       SetMenuItemType(eRadio);
     }
 
-    SetCheckState(mContent->AttrValueIs(kNameSpaceID_None, uWidgetAtoms::checked,
-                                       uWidgetAtoms::_true, eCaseMatters));
-    dbusmenu_menuitem_property_set_int(mDbusMenuItem,
-                                       DBUSMENU_MENUITEM_PROP_TOGGLE_STATE,
-                                       IsChecked() ?
-                                       DBUSMENU_MENUITEM_TOGGLE_STATE_CHECKED : 
-                                        DBUSMENU_MENUITEM_TOGGLE_STATE_UNCHECKED);
+    SyncStateFromContent();
 
   } else {
     dbusmenu_menuitem_property_remove(mDbusMenuItem,
@@ -546,32 +586,49 @@ uGlobalMenuItem::Refresh(uMenuObjectRefreshMode aMode)
 
     nsIDocument *doc = mContent->GetCurrentDoc();
     if (doc) {
-      nsAutoString value;
-      mContent->GetAttr(kNameSpaceID_None, uWidgetAtoms::command, value);
-      if (!value.IsEmpty()) {
-        mCommandContent = doc->GetElementById(value);
-        if (mCommandContent) {
-          mListener->RegisterForContentChanges(mCommandContent, this);
+      nsCOMPtr<nsIDOMDocument> domDoc = do_QueryInterface(doc);
+      NS_ASSERTION(domDoc, "nsIDocument failed QI to nsIDOMDocument");
+      if (domDoc) {
+        nsAutoString value;
+        mContent->GetAttr(kNameSpaceID_None, uWidgetAtoms::command, value);
+        if (!value.IsEmpty()) {
+          nsCOMPtr<nsIDOMElement> elem;
+          domDoc->GetElementById(value, getter_AddRefs(elem));
+          if (elem) {
+            mCommandContent = do_QueryInterface(elem);
+            if (mCommandContent) {
+              mListener->RegisterForContentChanges(mCommandContent, this);
+            }
+          }
         }
-      }
-      mContent->GetAttr(kNameSpaceID_None, uWidgetAtoms::key, value);
-      if (!value.IsEmpty()) {
-        mKeyContent = doc->GetElementById(value);
-        if (mKeyContent) {
-          mListener->RegisterForContentChanges(mKeyContent, this);
+        mContent->GetAttr(kNameSpaceID_None, uWidgetAtoms::key, value);
+        if (!value.IsEmpty()) {
+          nsCOMPtr<nsIDOMElement> elem;
+          domDoc->GetElementById(value, getter_AddRefs(elem));
+          if (elem) {
+            mKeyContent = do_QueryInterface(elem);
+            if (mKeyContent) {
+              mListener->RegisterForContentChanges(mKeyContent, this);
+            }
+          }
         }
       }
     }
+  }
 
-    SyncLabelFromContent(mCommandContent);
-    SyncSensitivityFromContent(mCommandContent);
+  SyncStateFromCommand();
+  SyncLabelFromCommand();
+  SyncSensitivityFromCommand();
+
+  if (aMode == eRefreshFull) {
     SyncTypeAndStateFromContent();
     SyncAccelFromContent();
+    SyncLabelFromContent();
+    SyncSensitivityFromContent();
   }
 
   SyncVisibilityFromContent();
   SyncIconFromContent();
-  SyncStateFromCommand();
 }
 
 /*static*/ void
@@ -774,7 +831,7 @@ uGlobalMenuItem::Destroy()
 
   if (mDbusMenuItem) {
     guint found = g_signal_handlers_disconnect_by_func(mDbusMenuItem,
-                                                       FuncToVoidPtr(ItemActivatedCallback),
+                                                       uGlobalMenuUtils::FuncToVoidPtr(ItemActivatedCallback),
                                                        this);
     NS_ASSERTION(found == 1, "Failed to disconnect \"activated\" handler");
   }
@@ -802,18 +859,15 @@ uGlobalMenuItem::ObserveAttributeChanged(nsIContent *aContent,
         aAttribute == uWidgetAtoms::key) {
       Refresh(eRefreshFull);
     } else if (aAttribute == uWidgetAtoms::label) {
-      if (!mCommandContent) {
-        SyncLabelFromContent();
-      }
+      SyncLabelFromContent();
     } else if (aAttribute == uWidgetAtoms::accesskey) {
-      SyncLabelFromContent(mCommandContent);
+      SyncLabelFromContent();
     } else if (aAttribute == uWidgetAtoms::disabled) {
-      if (!mCommandContent) {
-        SyncSensitivityFromContent();
-      }
-    } else if (aAttribute == uWidgetAtoms::checked ||
-               aAttribute == uWidgetAtoms::type) {
+      SyncSensitivityFromContent();
+    } else if (aAttribute == uWidgetAtoms::type) {
       SyncTypeAndStateFromContent();
+    } else if (aAttribute == uWidgetAtoms::checked) {
+      SyncStateFromContent();
     } else if (aAttribute == uWidgetAtoms::image) {
       SyncIconFromContent();
     } else if (aAttribute == uWidgetAtoms::hidden ||
@@ -822,9 +876,11 @@ uGlobalMenuItem::ObserveAttributeChanged(nsIContent *aContent,
     }
   } else if (aContent == mCommandContent) {
     if (aAttribute == uWidgetAtoms::label) {
-      SyncLabelFromContent(mCommandContent);
+      SyncLabelFromCommand();
     } else if (aAttribute == uWidgetAtoms::disabled) {
-      SyncSensitivityFromContent(mCommandContent);
+      SyncSensitivityFromCommand();
+    } else if (aAttribute == uWidgetAtoms::checked) {
+      SyncStateFromCommand();
     }
   } else if (aContent == mKeyContent) {
     SyncAccelFromContent();
